unit ACQUIRE; {Unit 8 of CONGRESS.PAS}

interface

  uses
    GLOBALS, MATHS, UTILS, ROOTS, Graph, Dos, Crt;

  procedure InverseRootsToCoeffs (p: integer;
                  var mu, alpha: complexVector);

  procedure RootsAcquisition (var p, r: integer;
                  var mu: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

  procedure PolarAcquisition (var p, r: integer;
                  var mu: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

  procedure CoeffAcquisition (var g: integer;
                  var gamma: complexVector;
                  stability: boolean);

  procedure ParameterAcquisition (var p, r: integer;
                  var alpha: vector;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

  procedure RationalAcquisition (var ARMAGen: ARMARecord;
                  stability: boolean);

implementation

{_________________________________________________}

  procedure InverseRootsToCoeffs (p: integer;
                  var mu, alpha: complexVector);

    var
      j, k: integer;
      store: complex;


  begin {RootToCoefficient}
    alpha[0].re := 1.0;
    alpha[0].im := 0.0;

    for k := 1 to p do
      begin {k}
        alpha[k].im := 0.0;
        alpha[k].re := 0.0;
        for j := k downto 1 do
          begin {j}
            ComplexMult(mu[k], alpha[j - 1], store);
            ComplexSub(alpha[j], store, alpha[j]);
          end; {j}
      end; {k}

  end; {InverseRootsToCoeffs}

  {_________________________________________________}

  procedure RootsAcquisition (var p, r: integer;
                  var mu: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

    const
      virtualZero = 1E-12;

    var
      i: integer;
      pi, radToDeg, modulus: real;

  begin {Roots Acquisition}
    pi := 4 * ArcTan(1.0);
    radToDeg := 180 / pi;

    Writeln('    What is the order of the polynomial?');
    GetBoundedInteger(0, 25, p);
    Writeln;

    r := 0;
    if p > 1 then
      repeat
        Writeln('    How may pairs of complex conjugates?');
        GetBoundedInteger(0, 25, r);
      until (2 * r <= p) and (r >= 0);

    i := 1;
    while (i <= p) do
      begin {while}
        if (i = 1) and (r > 0) then
          Writeln('    Complex conjugate roots')
        else if (i = 2 * r + 1) then
          Writeln('    Real roots');
        repeat
          Writeln('    Specify  root[', i : 1, '].re');
          ReadBoundedNumber(-1.0, 1.0, mu[i].re);
          if i <= 2 * r then
            begin
              Writeln('    Specify  root[', i : 1, '].im');
              ReadBoundedNumber(-1.0, 1.0,mu[i].im);
              mu[i + 1].re := mu[i].re;
              mu[i + 1].im := -mu[i].im;
            end;
          modulus := mu[i].re * mu[i].re + mu[i].im * mu[i].im;
          if (stability and (modulus >= 1)) then
            begin
              Sysbeep(3);
              Writeln('    The modulus of the root must be less than unity');
            end;
        until (modulus < 1) or (stability = false);
        if i <= 2 * r then
          i := i + 2
        else
          i := i + 1;
      end; {while}

    Writeln;
    for i := 1 to p do
      begin
        Writeln('    root(', i : 1, ').re = ', mu[i].re : 8 : 4);
        Writeln('    root(', i : 1, ').im = ', mu[i].im : 8 : 4);
        Writeln;
      end;
    Writeln;

    Writeln('    Press <RETURN> to continue');
    Readln;

    for i := 1 to p do
      begin {i}
        polarRoot[i].modu := Sqrt(Sqr(mu[i].re) + Sqr(mu[i].im));
        if Abs(mu[i].re) < virtualZero then
          begin
            if mu[i].im < 0 then
              polarRoot[i].argu := pi / 2
            else
              polarRoot[i].argu := -pi / 2
          end
        else if mu[i].re < 0 then
          polarRoot[i].argu := ArcTan(mu[i].im / mu[i].re) + pi
        else
          polarRoot[i].argu := ArcTan(mu[i].im / mu[i].re);
        polarRoot[i].argu := polarRoot[i].argu * radToDeg;
      end; {i}

    Writeln;
    for i := 1 to p do
      begin
        Writeln('    root(', i : 1, ').modu = ', polarRoot[i].modu : 8 : 4);
        Writeln('    root(', i : 1, ').argu = ', polarRoot[i].argu : 8 : 4);
        Writeln;
      end;
    Writeln;

    Writeln('    Press <RETURN> to continue');
    Readln;

  end; {RootsAcquisition}

   {_________________________________________________}

  procedure PolarAcquisition (var p, r: integer;
                  var mu: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

   {Revised by D.S.G.P., 5th September 1996}

    const
      virtualZero = 1E-12;

    var
      i: integer;
      degToRad, rho, theta: real;

  begin {Roots Acquisition}
    degToRad := ArcTan(1.0) / 45;

    Writeln('    What is the order of the polynomial?');
   {Readln(p);}
    GetBoundedInteger(0, 25, p);
    Writeln;

    r := 0;
    if p > 1 then
      repeat
        Writeln('    How may pairs of complex conjugates?');
        GetBoundedInteger(0, 25, r);
      until (2 * r <= p) and (r >= 0);

    i := 1;
    while (i <= p) do
      begin {while}
        if (i = 1) and (r > 0) then
          Writeln('    Complex conjugate roots')
        else if (i = 2 * r + 1) then
          Writeln('    Real roots');
        repeat
          if i <= 2 * r then
            begin
              Writeln('    Specify  the argument theta[', i : 1, '] in degrees');
              ReadBoundedNumber(0.0, 180.0,polarRoot[i].argu);
              Writeln('    Specify  the modulus rho[', i : 1, ']');
              ReadBoundedNumber(0.0, 1.0, polarRoot[i].modu);
              if polarRoot[i + 1].modu < 0 then
                polarRoot[i + 1].modu := -polarRoot[i + 1].modu;
              polarRoot[i + 1].argu := -polarRoot[i].argu;
              polarRoot[i + 1].modu := polarRoot[i].modu;
            end
          else {if  i > 2 * r }
            begin
              Writeln('    Specify  the real root: root[', i : 1, ']');
              ReadBoundedNumber(0.0, 1.0, polarRoot[i].modu);
              polarRoot[i].argu := 0.0;
            end;
          if (stability and (sqr(polarRoot[i].modu) >= 1)) then
            begin
              Sysbeep(3);
              Writeln('    The modulus of the root must be less than unity');
            end;
        until (polarRoot[i].modu < 1) or (stability = false);
        if i <= 2 * r then
          i := i + 2
        else
          i := i + 1;
      end; {while}

    Writeln;
    for i := 1 to p do
      begin
        Writeln('    rho(', i : 1, ') = ', polarRoot[i].modu : 8 : 4);
        Writeln('    theta(', i : 1, ') = ', polarRoot[i].argu : 8 : 4);
        Writeln;
      end;
    Writeln;

    Writeln('    Press <RETURN> to continue');
    Readln;

    for i := 1 to p do
      begin {i}
        if Abs(polarRoot[i].argu) < virtualZero then
          begin {if}
            mu[i].re := polarRoot[i].modu;
            mu[i].im := 0.0;
          end {if}
        else
          begin {else}
            theta := polarRoot[i].argu * degToRad;
            rho := polarRoot[i].modu;
            mu[i].re := rho * Cos(theta);
            mu[i].im := rho * Sin(theta);
          end; {else}
      end; {i}

    for i := 1 to p do
      begin
        Writeln('    root(', i : 1, ').re = ', mu[i].re : 8 : 4);
        Writeln('    root(', i : 1, ').im = ', mu[i].im : 8 : 4);
        Writeln;
      end;

    Writeln;
    Writeln('    Press <RETURN> to continue');
    Readln;

  end; {PolarAcquisition}

   {_________________________________________________}

  procedure CoeffAcquisition (var g: integer;
                  var gamma: complexVector;
                  stability: boolean);

    var
      i: integer;
      stable: boolean;
      alpha: vector;

  begin
    Writeln('    What is the order of the polynomial?');
    GetBoundedInteger(0, 15, g);
    Writeln;
    Writeln('    Real coefficients');

    repeat
      Writeln;
      for i := 0 to g do
        begin
          Writeln('    Specify   gamma[', i : 1, ']');
          ReadANumber(gamma[i].re);
          gamma[i].im := 0;
          alpha[i] := gamma[i].re;
        end;
      if stability then
        begin {test for stability}
          JuryCriterion(alpha, g, stable);
          if not stable then
            begin
              SysBeep(3);
              Writeln('    The roots are unstable!');
              Writeln('    Respecify the coefficients.');
            end;
        end; {test for stability}
    until (stable or (not stability));

  end; {CoeffAcquisition}

{_________________________________________________}

  procedure ParameterAcquisition (var p, r: integer;
                  var alpha: vector;
                  var cartRoot: complexVector;
                  var polarRoot: polarVector;
                  stability: boolean);

    var
      i: integer;
      polarSpec, cartSpec, coeffSpec: boolean;
      complexAlpha: complexVector;

   {Revised by D.S.G.P., 16th August 1996}

  begin
    complexAlpha[0].re := 1;

    polarSpec := false;
    cartSpec := false;
    coeffSpec := false;

    YesOrNo('    Do you want to specify the auxiliary polynomial in polar form', polarSpec);
    if polarSpec = false then
      YesOrNo('    Do you want to specify in Cartesian form', cartSpec);
    if (polarSpec = false) and (cartSpec = false) then
      begin
        coeffSpec := true;
        Writeln('    You have chosen to specify the polynomial coefficients');
      end;

    if polarSpec then
      PolarAcquisition(p, r, cartRoot, polarRoot, stability)
    else if cartSpec then
      RootsAcquisition(p, r, cartRoot, polarRoot, stability)
    else if coeffSpec then
      CoeffAcquisition(p, complexAlpha, stability);

    if coeffSpec then
      FindRoots(p, complexAlpha, cartRoot, polarRoot)
    else
      InverseRootsToCoeffs(p, cartRoot, complexAlpha);

    for i := 0 to p do
      alpha[i] := complexAlpha[i].re;

  end; {ParameterAcquisition}

{_________________________________________________}

  procedure RationalAcquisition (var ARMAGen: ARMARecord;
                  stability: boolean);

    var
      i: integer;
      poleQ, zeroQ: boolean;

  begin
    ClrScr;

    Writeln;
    Writeln('    SPECIFY AN AUTOREGRESSIVE MODEL');
    Writeln;
    ARMAGen.p := 0;
    ARMAGen.q := 0;
    stability := true;
    YesOrNo('    Are there any poles', poleQ);
    if poleQ then
      ParameterAcquisition(ARMAGen.p, ARMAGen.pr, ARMAGen.alpha, ARMAGen.poles, ARMAGen.polarPoles, stability)
    else
      ARMAGen.alpha[0] := 1;
    Writeln;
    stability := false;
    YesOrNo('    Are there any zeros', zeroQ);
    if zeroQ then
      ParameterAcquisition(ARMAGen.q, ARMAGen.qr, ARMAGen.mu, ARMAGen.zeros, ARMAGen.polarZeros, stability)
    else
      ARMAGen.mu[0] := 1;

   {Print out the acquired coefficients}
    Writeln('    Denominator Coefficients');
    Writeln;
    for i := 0 to ARMAGen.p do
      Writeln('    alpha(', i : 1, ') = ', ARMAGen.alpha[i] : 8 : 4);

    Writeln;
    Writeln('    Numerator Coefficients');
    Writeln;
    for i := 0 to ARMAGen.q do
      Writeln('    mu(', i : 1, ') = ', ARMAGen.mu[i] : 8 : 4);

    Writeln;
    Writeln('    Press <RETURN> to continue');
    Readln;

    ARMAmodel := true;
    ARMAGen.bandLimits := false;
    ARMAGen.rootsFound := true;
    ARMAGen.parametersFound := true;
    ARMAGen.varepsilon := 1.0;
    ARestimates := false;

  end; {RationalAcquisition}

  {_________________________________________________}

end. {ACQUIRE: Unit 8 of CONGRESS.PAS}

