program CONGRESS;

  uses
    GLOBALS, MATHS, UTILS, ACQUIRE, AUTOREG, ORGANISE, Dos, Crt, Graph;

  var
    theMenuItem: integer;

begin
  SetTheText;

  Writeln;
  Writeln('     CONGRESS.PAS: Frequency-Limited ARMA Models');
  Writeln;
  Writeln('     This program  can be used to investigate  band-limited autoregressive');
  Writeln('     processes, of which the  spectral density  functions are supported in');
  Writeln('     the  frequency domain on  an interval that  runs from  zero frequency');
  Writeln('     up to  a limiting value that  is  less  than the Nyquist frequency of');
  Writeln('     pi radians per sampling interval.');
  Writeln;
  Writeln('     The effects can be investigated  of ignoring the frequency limitation');
  Writeln('     by employing a conventional  procedure to  estimate the parameters of');
  Writeln('     the band-limited autregressive process.');
  Writeln;
  Writeln('     Appropriate  estimates  may be derived by  synthesising  a continuous');
  Writeln('     function  from the Fourier coefficients of the data and by resampling');
  Writeln('     the function at less  than the  original sampling  rate. In this way,');
  Writeln('     the resampled data can be  mapped onto the Nyquist interval, and then');
  Writeln('     an ordinary estimator can be used.');
  Writeln;
  Writeln(' ' : 66, '<RETURN>');
  Readln;

  graphicsOn := false;
  noSCREENBOX := true;
  SCREENBOX.XC := 0;
  SCREENBOX.YC := 0;

  allsDone := false;
  noPageParameters := true;
  stability := true;
  thereIsData := false;
  pseudoData := false;
  empiricData := false;
  trendEstimated := false;
  SubsampledData := false;
  ARMAModel := false;
  ARestimates := false;
  PreSpecifiedZeros := false;
  ARMAGen.bandLimits := false;
  ARMAGen.contaminated := false;
  slowOrFast := fast;
  figNumber := 1;
  genSeed := TickCount mod 714025;
  seedStore := genSeed;

  repeat
    ClrScr;

    Writeln;
    Writeln('    CONGRESS.PAS: Frequency-Limited  ARMA Processes');
    Writeln;
    Writeln('         1. Page Parameters');
    Writeln;
    Writeln('         2. Specify an ARMA Model');
    Writeln;
    Writeln('         3. Get the Data, Plot the Data, Transform the Data');
    Writeln;
    Writeln('         4. Polynomial Detrending and Lowpass Filtering');
    Writeln;
    Writeln('         5. Plot the Periodogram');
    Writeln;
    Writeln('         6. ARMA Estimation');
    Writeln;
    Writeln('         7. Plot the Spectral Density');
    Writeln;
    Writeln('         8. Analysis of ARMA Models');
    Writeln;
    Writeln('         9. SAVE and EXIT');
    Writeln;
    Writeln('                      Select a number and press <RETURN>');

    GetBoundedInteger(1, 9, theMenuItem);

    case theMenuItem of
      1: 
        PageSetUp(SCREENBOX, POSTBOX, PSPict, TeXFormat);{in UTILS}
      2: 
        begin
          RationalAcquisition(ARMAGen, stability);
          ARMAGen.varepsilon := 1.0;
        end;
      3: {in ORGANISE}
        GetTheData(dataBOX, SCREENBOX, POSTBOX, dataVector); {in ORGANISE}
      4: {in ORGANISE}
        TrendExtract(SCREENBOX, POSTBOX, dataBOX, trendBOX, residueBOX, dataVector, trendVector, residueVec);
      5:  {in ORGANISE}
        PlotThePeriodogram(ARMAGen, ARMAEst, dataBOX, SCREENBOX, POSTBOX, dataVector);
      6:  {in ORGANISE}
        AREstimation(ARMAGen, ARMAEst, dataBOX, dataVector); {in ORGANISE}
      7:  {in ORGANISE}
        PlotTheSpectrum(ARMAGen, ARMAEst, dataBOX, SCREENBOX, POSTBOX, dataVector);
      8: {in AUTOREG}
        ModelAnalysis(ARMAGen, ARMAEst, dataBOX);
      9: {in ORGANISE}
        SaveAndExit(dataBOX, dataVector, ARMAGen, ARMAEst);
    end;

  until allsDone;
  RevertText;

end.  {CONGRESS Main Unit}

{________________________________________________}

   {Compile the Units in the Following Order:}

   {1. GLOBALS}
   {2. MATHS}
   {3. UTILS}
   {4. SCREEN}
   {5. POST}
   {6. ROOTS}
   {7. SPECTRAL}
   {8. ACQUIRE}
   {9. AUTOREG}
   {10. DATA}
   {11. ORGANISE}

